# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from oslo_log import log as logging
from solumclient.v1 import workflow as cli_wf
import yaml

from solumdashboard.api.client import client as solumclient


LOG = logging.getLogger(__name__)


class ScaleForm(forms.SelfHandlingForm):
    target = forms.IntegerField(label=_("Target"))

    def handle(self, request, data):
        app_id = self.initial.get('application_id')
        LOG.info('ScaleApplication %s' % data)

        if data["target"] <= 0:
            exceptions.handle(self.request,
                              _("Scale target must be greater than zero"))

        solum = solumclient(request)
        actions = ['scale']
        cli_wf.WorkflowManager(solum, app_id=app_id).create(
            actions=actions)

        return True


class CreateForm(forms.SelfHandlingForm):
    source = forms.ChoiceField(
        label=_('Source'),
        choices=[
            ('app_file', _('App File')),
            ('input', _('Input'))
        ],
        widget=forms.Select(attrs={
            'class': 'switchable',
            'data-slug': 'source'
        })
    )
    app_file = forms.FileField(
        label=_("Local app file location"),
        required=False,
        widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-switch-on': 'source',
            'data-source-app_file': _('Local app file location')
        })
    )
    name = forms.CharField(label=_("Application Name"), required=False,
                           max_length=100)
    languagepack = forms.CharField(label=_("Languagepack"), required=False)
    git_url = forms.URLField(label=_("Source repository"), required=False)
    run_cmd = forms.CharField(label=_("Application entry point"),
                              required=False)
    unittest_cmd = forms.CharField(label=_(
        "Command to execute unit tests"), required=False)
    port = forms.IntegerField(label=_("The port your application listens on"),
                              min_value=0,
                              required=False)
    param_file = forms.FileField(label=_(
        "A yaml file containing custom parameters"), required=False)

    def clean(self):
        cleaned_data = super(CreateForm, self).clean()
        import_type = cleaned_data.get('source')
        if import_type == 'app_file' and not cleaned_data.get('app_file'):
            msg = _('Please supply an app file')
            raise forms.ValidationError(msg)
        elif import_type == 'input':
            if not (cleaned_data.get('name')):
                msg = _('Please supply a name')
                raise forms.ValidationError(msg)
            elif not cleaned_data.get('languagepack'):
                msg = _('Please supply a languagepack')
                raise forms.ValidationError(msg)
            elif not cleaned_data.get('git_url'):
                msg = _('Please supply a github url')
                raise forms.ValidationError(msg)
            elif not cleaned_data.get('run_cmd'):
                msg = _('Please supply a run command')
                raise forms.ValidationError(msg)
        return cleaned_data

    def handle(self, request, data):
        LOG.info('CreateApplication %s' % data)
        solum = solumclient(request)

        app_data = None
        if data['source'] == 'app_file':
            inf = data['app_file'].read()
            app_data = yaml.load(inf)
            if 'repo_token' not in app_data:
                app_data['repo_token'] = ''
        else:
            app_data = {
                'version': 1,
                'description': 'default app description',
                'source': {
                    'repository': '',
                    'revision': 'master',
                    'repo_token': ''
                },
                'workflow_config': {
                    'test_cmd': '',
                    'run_cmd': ''
                }
            }

        if data['name']:
            app_data['name'] = data['name']

        if data['languagepack']:
            app_data['languagepack'] = data['languagepack']

        if data['git_url']:
            app_data['source'] = dict()
            app_data['source']['repository'] = data['git_url']
            app_data['source']['revision'] = 'master'

        if data['run_cmd']:
            if app_data.get('workflow_config') is None:
                app_data['workflow_config'] = dict()
            if not app_data['workflow_config']['run_cmd']:
                app_data['workflow_config']['run_cmd'] = data['run_cmd']

        if data['unittest_cmd']:
            if app_data.get('workflow_config') is None:
                app_data['workflow_config'] = dict()
            if not app_data['workflow_config']['test_cmd']:
                app_data['workflow_config']['test_cmd'] = data['unittest_cmd']

        if not app_data.get('ports'):
            app_data['ports'] = []
            if data['port']:
                app_data['ports'].append(data['port'])
            else:
                app_data['ports'].append(80)

        if data['param_file']:
            param_def = data['param_file'].read()
            app_data['parameters'] = yaml.load(param_def)

        try:
            solum.apps.create(**app_data)
            messages.success(request,
                             _('Application was successfully created.'))
            return True
        except Exception:
            msg = _('Unable to create application.')
            redirect = reverse("horizon:solum:applications:index")
            exceptions.handle(request, msg, redirect=redirect)
            return False


class LaunchForm(forms.SelfHandlingForm):
    du_id = forms.CharField(label=_("ID of the DU image"), required=False)

    def handle(self, request, data):
        app_id = self.initial.get('application_id')
        LOG.info('LaunchApplication %s' % data)
        solum = solumclient(request)

        if data["du_id"]:
            actions = ['deploy']
            cli_wf.WorkflowManager(
                solum, app_id=app_id).create(
                    actions=actions, du_id=data["du_id"])
        else:
            actions = ['unittest', 'build', 'deploy']
            cli_wf.WorkflowManager(
                solum, app_id=app_id).create(
                    actions=actions)

        return True



# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Mistral DSL and Heat Template parsing and representation routines.

This code does not import the original YAML representations of the Mistral DSL
and Heat Templates as the data may be in memory instead of file.  PyYAML is a
good library to load the YAML and translate into a Python dictionary with code
like:

import yaml

with open("mistral_dsl.yaml", "r") as fptr
    data = yaml.safe_load(fptr)

Important note:
This code expects that the loading code performs some basic YAML validation
* Mistral DSLs must include a "Workflow" section with a "tasks" subsection
* Mistral DSLs must have one task with no on-success (last task)
* Heat Templates must include a "requirements" section
"""


def get_mistral_tasks(data, start_task_name=None):
    """Returns an ordered Mistral task list from a DSL."""
    task_list = []
    task_dict = data["Workflow"]["tasks"]
    for key, task in task_dict.items():
        on_success = task.get("on-finish", task.get("on-success"))
        on_error = task.get("on-finish", task.get("on-error"))
        task_list.append([key, on_success, on_error])
    curr_task_name = None
    sorted_task_list = []
    no_suc_list = ([[name, on_suc, on_err] for (name, on_suc, on_err) in
                    task_list if on_suc is None])
    sorted_task_list.insert(0, no_suc_list[0])
    curr_task_name = no_suc_list[0][0]
    for count in range(len(task_list) - 1):
        for task in task_list:
            task_name, on_success, on_error = task
            if on_success == curr_task_name:
                curr_task_name = task_name
                sorted_task_list.insert(0, task)
                break
        if start_task_name:
            if start_task_name == task_name:
                break
    return sorted_task_list


def create_svg_mistral_tasks(task_list, radius=45):
    """Create an SVG UI diagram of Mistral task flow.

    This takes the output of get_mistral_tasks() and generates an SVG-based
    graphical diagram of the ordered Mistral tasks.  The code automatically
    scales the SVG based on the circle radius value.  Note that SVG circles
    don't scale radius by percentages very well which is why this is still
    pixel math.  The circle radius is the diagonal length of the viewport
    which is not very useful in this case.
    """
    indent = radius * 1.1
    diameter = radius * 2
    num_tasks = len(task_list)
    if num_tasks < 1:
        return "[No Tasks Found]"
    svg_output = ("<svg height=\"%d\" width=\"%d\">\n" %
                  ((diameter * 1.10), ((num_tasks-1) * diameter * 1.3) +
                   indent * 2))
    svg_output += ("  <line x1=\"%d\" y1=\"50%%\" x2=\"%d\" y2=\"50%%\" style="
                   "\"stroke:rgb(0,0,0);stroke-width:3\"/>\n" %
                   (indent, ((num_tasks-1) * diameter * 1.2) + indent))
    svg_output += ("  <g stroke=\"black\" stroke-width=\"3\" fill="
                   "\"lightgrey\">\n")
    for counter in range(num_tasks):
        svg_output += ("    <circle cx=\"%d\" cy=\"50%%\" r=\"%d\"/>\n" %
                       ((counter * diameter * 1.2 + indent), radius))
    svg_output += "  </g>\n"
    svg_output += "  <g style=\"text-anchor: middle; font-size: 13px\">\n"
    for counter in range(num_tasks):
        svg_output += ("    <text x=\"%d\" y=\"55%%\">%s</text>\n" %
                       ((counter * diameter * 1.2 + indent),
                        task_list[counter][0]))
    svg_output += "  </g>\n"
    svg_output += "</svg>\n"
    return svg_output


def get_mistral_required_input(data, start_task_name=None):
    """Returns required Mistral DSL user input field information.

    Note that this code ignores Mistral DSL values that are enumerated in the
    ignore_list list below which are under the "parameters:" label.  The
    recommendation is to not nest sections under a "parameters:" label and
    just list name/value pairs in the parameters section.
    """
    input_dict = {}
    task_list = get_mistral_tasks(data, start_task_name)
    task_key_list = [item[0] for item in task_list]
    task_dict = data["Workflow"]["tasks"]
    ignore_list = ["params", "settings", "arguments"]
    publish_list = []
    for task in task_key_list:
        param_list = task_dict[task].get("parameters", [])
        publish_list.extend(task_dict[task].get("publish", []))
        for param in param_list:
            if param not in ignore_list and param not in publish_list:
                if param not in input_dict:
                    input_dict[param] = [task]
                else:
                    input_dict[param].append(task)
    return input_dict


def get_heat_required_input(data):
    """Returns Heat required user input fields."""
    heat_params = []
    heat_param_dict = data["parameters"]
    for key, heat_param in heat_param_dict.items():
        heat_params.append([key,
                            heat_param.get("type"),
                            heat_param.get("default"),
                            heat_param.get("description")])
    return sorted(heat_params)


if __name__ == "__main__":
    # This code is left for example and basic testing purposes; it is not
    # intended for production use.

    import yaml

    with open("dsl.yaml", "r") as FPTR:
        DATA = yaml.safe_load(FPTR)
    print("\nMistral Task workflow:")
    RET_DICT = get_mistral_tasks(DATA)
    for VAL in RET_DICT:
        print("%s - success: %s, error: %s" % (VAL[0], VAL[1], VAL[2]))

    SVG_TEXT = create_svg_mistral_tasks(RET_DICT, 45)
    print("\nSVG task output:\n" + SVG_TEXT)

    # Mistral required input
    print("\nMistral required inputs:")
    RET_DICT = get_mistral_required_input(DATA)
    for KEY, VAL in RET_DICT.items():
        print(KEY, "", VAL)

    # Heat required input
    print("\nHeat required inputs:")
    with open("hot.yaml", "r") as FPTR:
        DATA = yaml.safe_load(FPTR)
    RET_DICT = get_heat_required_input(DATA)
    for VAL in RET_DICT:
        print("%s - %s, %s, %s" % (VAL[0], VAL[1], VAL[2], VAL[3]))



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings

from horizon import exceptions

from solumclient import client as api_client

from openstack_dashboard.api import base
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


def get_solum_url(request):
    endpoint = getattr(settings, 'SOLUM_URL', None)
    if not endpoint:
        try:
            endpoint = base.url_for(request, 'application_deployment')
        except exceptions.ServiceCatalogException:
            endpoint = 'http://localhost:9777'
            LOG.warning('Solum API location could not be found in Service '
                        'Catalog, using default: {0}'.format(endpoint))
    return endpoint


def client(request):
    auth_url = getattr(settings, 'OPENSTACK_KEYSTONE_URL')
    return api_client.Client(1, endpoint=get_solum_url(request),
                             token=request.user.token.id,
                             auth_url=auth_url)



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from oslo_log import log as logging
import yaml

from solumdashboard.api.client import client as solumclient


LOG = logging.getLogger(__name__)


class CreateForm(forms.SelfHandlingForm):
    source_uri = forms.CharField(label=_("Source URI"))
    name = forms.CharField(label=_("Languagepack Name"), max_length=100)
    description = forms.CharField(label=_("Description"), required=False,
                                  max_length=255)
    param_file = forms.FileField(label=_("Parameter File"),
                                 required=False)
    lp_metadata = forms.FileField(label=_("Languagepack Metadata File"),
                                  required=False)

    def handle(self, request, data):
        LOG.info('CreateLanguagepack %s' % data)
        solum = solumclient(request)

        param_data = {}
        if data['param_file']:
            inf = data['param_file'].read()
            param_data = yaml.load(inf)

        lp_metadata = None

        if data['lp_metadata']:
            lp_metadata = json.dumps(json.load(data['lp_metadata']))

        try:
            solum.languagepacks.create(
                name=data['name'], source_uri=data['source_uri'],
                lp_metadata=lp_metadata, lp_params=param_data,
                description=data['description'])
            message = _(
                'Languagepack %s was successfully created.') % data['name']
            messages.success(request, message)
        except Exception:
            redirect = reverse('horizon:solum:languagepacks:index')
            exceptions.handle(
                self.request,
                _('Unable to create languagepack.'),
                redirect=redirect)

        return True



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from solumdashboard.api.client import client as solumclient


class CreateAssembly(tables.LinkAction):
    name = "create"
    verbose_name = _("New Assembly")
    url = "horizon:solum:assemblies:create"
    classes = ("btn-launch", "ajax-modal")


class DeleteAssembly(tables.DeleteAction):

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Assembly",
            u"Delete Assemblies",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Scheduled deletion of Assembly",
            u"Scheduled deletion of Assemblies",
            count
        )

    def allowed(self, request, template):
        return True

    def action(self, request, assembly_id):
        solum = solumclient(request)
        solum.assemblies.delete(assembly_id=assembly_id)


class ViewAssembly(tables.LinkAction):
    name = "view"
    verbose_name = _("View")
    url = "horizon:solum:assemblies:detail"
    classes = ("btn-edit",)


class AssembliesTable(tables.DataTable):
    uuid = tables.Column('uuid', verbose_name=_('UUID'),
                         link=("horizon:solum:assemblies:detail"))
    name = tables.Column('name', verbose_name=_('Name'))
    application = tables.Column('application', verbose_name=_('Application'),
                                link=("horizon:solum:applications:detail"))
    description = tables.Column('description', verbose_name=_('Description'))

    def get_object_id(self, app):
        return app.uuid

    class Meta(object):
        name = "assemblies"
        verbose_name = _("Assemblies")
        table_actions = (DeleteAssembly,)
        row_actions = (ViewAssembly, DeleteAssembly)



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import forms
from horizon import workflows

from solumdashboard.api.client import client as solumclient


class UpdateApplicationAction(workflows.Action):
    name = forms.CharField(label=_("Application Name"), required=False)
    description = forms.CharField(label=_("Description"), required=False)
    languagepack = forms.CharField(label=_("Languagepack"), required=False)
    ports = forms.IntegerField(label=_("Ports"), required=False)

    source = {}
    source["repository"] = forms.CharField(label=_("Source Repository"),
                                           required=False)
    source["revision"] = forms.CharField(label=_("Source Revision"),
                                         required=False)

    workflow_config = {}
    workflow_config["test_cmd"] = forms.CharField(
        label=_("Workflow Test Command"), required=False)
    workflow_config["run_cmd"] = forms.CharField(
        label=_("Workflow Run Command"), required=False)


class UpdateApplication(workflows.Step):
    action_class = UpdateApplicationAction
    depends_on = ("application_id",)
    contributes = ("name", "description", "languagepack", "ports", "source",
                   "workflow_config")


class UpdateApplicationClass(workflows.Workflow):
    slug = "update_app"
    name = _("Update App")
    finalize_button_name = _("Update")
    success_message = _("App updated")
    failure_message = _("Could not update app")
    success_url = "horizon:solum:applications:index"
    default_steps = (UpdateApplication,)
    depends_on = ("application_id",)
    contributions = ("name", "description", "languagepack", "ports", "source",
                     "workflow_config")

    def handle(self, request, data):
        to_update = {}
        for field in data:
            if data[field] and field != 'application_id':
                to_update[field] = data[field]

        solum = solumclient(request)
        solum.apps.patch(data["application_id"], data=to_update)

        return True
